from typing import Any, Dict, List
from urllib.parse import urljoin

from requests import Session

from sourcestack.resource import Resource

type Job = Dict[str, Any]


# https://sourcestack.co/docs/example-queries/#jobs
class Jobs(Resource):
    def all(self) -> List[Job]:
        """
        Fetches all jobs from the SourceStack API.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get()

    def by_name(self, name: str, exact: bool = False) -> List[Job]:
        """
        Fetches jobs by name from the SourceStack API.

        Args:
            name (str): The name of the job.
            exact (bool): Whether to match the name exactly.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get(name=name, exact="true" if exact else "false")

    def by_parent(self, parent: str) -> List[Job]:
        """
        Fetches jobs by parent from the SourceStack API.

        Args:
            parent (str): The parent of the job.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get(parent=parent)

    def by_url(self, url: str) -> List[Job]:
        """
        Fetches jobs by url from the SourceStack API.

        Args:
            url (str): The url of the job.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get(url=url)

    def by_uses_product(self, uses_product: str, exact: bool = True) -> List[Job]:
        """
        Fetches jobs by product from the SourceStack API.

        Args:
            product (str): The product of the job.
            exact (bool): Whether to match the product exactly.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get(uses_product=uses_product, exact="true" if exact else "false")

    def by_uses_category(self, uses_category: str, exact: bool = True) -> List[Job]:
        """
        Fetches jobs by category from the SourceStack API.

        Args:
            category (str): The category of the job.
            exact (bool): Whether to match the category exactly.

        Returns:
            List[Dict[str, Any]]: A list of jobs.
        """
        return self._get(
            uses_category=uses_category, exact="true" if exact else "false"
        )

    def _get(self, **kwargs) -> List[Job]:
        url = urljoin(self.base_url, "jobs")
        response = self.session.get(url, params=kwargs)
        response.raise_for_status()
        return response.json()



from sourcestack.client import Client

client = Client(api_key="fake-api-key")


def test_client_api_key():
    assert client.api_key == "fake-api-key"


def test_client_base_url():
    assert client.base_url == "https://sourcestack-api.com"


def test_client_session():
    assert client.session.headers["X-API-KEY"] == "fake-api-key"



from requests import Session


class Resource:
    session: Session
    base_url: str

    def __init__(
        self,
        session: Session,
        base_url: str,
    ):
        """
        Initializes with a client and a base_url.

        Args:
            session (requests.Session): A session to use.
            base_url (str): The base url of the SourceStack API.

        """
        self.session = session
        self.base_url = base_url



from setuptools import find_packages, setup

VERSION = "0.0.5"
DESCRIPTION = "A python package for using the https://sourcestack.co/ API."

with open("README.md", "r", encoding="utf-8") as file:
    LONG_DESCRIPTION = file.read()
    LONG_DESCRIPTION_CONTENT_TYPE = "text/markdown"

setup(
    name="sourcestack",
    version=VERSION,
    description=DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    long_description_content_type=LONG_DESCRIPTION_CONTENT_TYPE,
    author="Kevin Sylvestre",
    author_email="kevin@ksylvest.com",
    license="MIT",
    packages=find_packages(),
    install_requires=["requests"],
    keywords="sourcestack",
    classifiers=[
        "Development Status :: 3 - Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
    ],
)
